/****************************************************************************
 * Copyright (C) 2009-2016    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Théodore Efremov  contact address: XMAILX                *
 *
 *                                                                           *
 * Creation Date  : 10/2024                                                  *
 * Last update    : 10/2024                                                  *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This is the analysis of 2024 run e_850 with a focus on the 2alpha channel*
 *                                                                           *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *****************************************************************************/

#include<iostream>
using namespace std;
#include"Analysis.h"
#include"NPAnalysisFactory.h"
#include"NPDetectorManager.h"
#include"NPOptionManager.h"
#include"RootOutput.h"
#include"RootInput.h"
////////////////////////////////////////////////////////////////////////////////
Analysis::Analysis(){
}
////////////////////////////////////////////////////////////////////////////////
Analysis::~Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::Init(){

  PISTA = (TPISTAPhysics*) m_DetectorManager->GetDetector("PISTA");
  FPMW  = (TFPMWPhysics*) m_DetectorManager->GetDetector("FPMW");
  IC  = (TICPhysics*) m_DetectorManager->GetDetector("IC");
  EXOGAM  = (TExogamPhysics*) m_DetectorManager->GetDetector("Exogam");
  Tracking = new TVamosReconstruction();
  Tracking->ReadMatrix("MRec.mat");


  // Energy loss table/cos(FPMW->Theta_in)
  Be10C = EnergyLoss("EnergyLossTable/Be10_C.G4table","G4Table",100);
  C12C = EnergyLoss("EnergyLossTable/C12_C.G4table","G4Table",100);
  C12Si = EnergyLoss("EnergyLossTable/C12_Si.G4table","G4Table",100);
  C12Al = EnergyLoss("EnergyLossTable/C12_Al.G4table","G4Table",100);
  U238C = EnergyLoss("EnergyLossTable/U238_C.G4table","G4Table",100);
  AlphaC = EnergyLoss("EnergyLossTable/alpha_C.G4table","G4Table",100);
  AlphaSi = EnergyLoss("EnergyLossTable/alpha_Si.G4table","G4Table",100);
  AlphaAl = EnergyLoss("EnergyLossTable/alpha_Al.G4table","G4Table",100);
  geloss_C12C   = new TGraph("EnergyLossTable/C12_C.dat");
  geloss_C12Al  = new TGraph("EnergyLossTable/C12_Al.dat");
  geloss_Be10C  = new TGraph("EnergyLossTable/Be10_C.dat");
  geloss_Be10Al = new TGraph("EnergyLossTable/Be10_Al.dat");



  m_BeamEnergy = 5.96738*238;
  m_XTarget_offset = 0;
  m_YTarget_offset = 0;
  m_ZTarget_offset = 0;
  m_Beam_ThetaX = 0;
  m_Beam_ThetaY = 0;



  InitInputBranch();
  InitOutputBranch();
  Rand = TRandom3();
  LoadCalibParameter();
  ReadAnalysisConfig();
  LoadTimeOffset();

  TargetThickness = 0.44*micrometer;

  m_BeamEnergy = U238C.Slow(m_BeamEnergy,0.5*TargetThickness,0);
  Transfer10Be = new NPL::Reaction("238U(12C,10Be)240Pu@1417");
  Transfer8Be  = new NPL::Reaction("238U(12C,8Be)242Pu@1417");
  Transfer14C  = new NPL::Reaction("238U(12C,14C)236U@1417");
  Elastic      = new NPL::Reaction("238U(12C,12C)238U@1417");

  Elastic->SetBeamEnergy(m_BeamEnergy);
  Transfer10Be->SetBeamEnergy(m_BeamEnergy);
  Transfer8Be->SetBeamEnergy(m_BeamEnergy);
  Transfer14C->SetBeamEnergy(m_BeamEnergy);

  C12  = new NPL::Particle("12C");
  Be10 = new NPL::Particle("10Be");

  chain = RootInput::getInstance()->GetChain();

  Xmean = 0;
  Ymean = 0;
  Xmean_iter = 0;
  Ymean_iter = 0;
  iteration = 0;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::LoadCalibParameter(){
  string input_path = "./Calibration/VAMOS/CHIO/charge.cal";

  ifstream ifile;
  ifile.open(input_path.c_str());
  string token;
  double p0, p1;
  int i=0;
  while(ifile>>token>>p0>>p1){
    cout << token << " " << p0 << " " << p1 << endl;
    m_Q_p0[i] = p0;
    m_Q_p1[i] = p1;
    i++;
  }
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::LoadTimeOffset(){
  ifstream ifile;
  // T13
  string filename = "./macro/AoQ/T13_offset.cal";
  ifile.open(filename.c_str());
  string token;
  double offset;
  int section=0;
  while(ifile>>token>>offset){
    m_T13_Offset[section] = offset;
    section++;
  }
  ifile.close();

  // T14
  filename = "./macro/AoQ/T14_offset.cal";
  ifile.open(filename.c_str());
  section=0;
  while(ifile>>token>>offset){
    m_T14_Offset[section] = offset;
    section++;
  }
  ifile.close();

}


////////////////////////////////////////////////////////////////////////////////
void Analysis::TreatEvent(){
  ReInitValue();

  VAMOS_TS_hour = fVAMOS_TS_sec/3600.;
  PISTA_TS_hour = fPISTA_TS_sec/3600.;

  VamosAnalysis();
  PistaAnalysis();
  ExogamAnalysis();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::ExogamAnalysis(){
  unsigned int size = EXOGAM->E_AB.size();

  //for(unsigned int i=0; i<size; i++){
  if(size==1){
    Exo_Theta = EXOGAM->Theta[0];
    Exo_Phi   = EXOGAM->Phi[0];
    Exo_E     = EXOGAM->E_AB[0];

    if(Exo_Theta!=-1000){
      if(FF_Theta!=-100){
        Exo_cosa  = sin(FF_Theta)*cos(FF_Phi)*sin(Exo_Theta)*cos(Exo_Phi) + sin(FF_Theta)*sin(FF_Phi)*sin(Exo_Theta)*sin(Exo_Phi) + cos(FF_Theta)*cos(Exo_Theta);
        //Exo_cosa  = sin(0)*cos(0)*sin(Exo_Theta)*cos(Exo_Phi) + sin(0)*sin(0)*sin(Exo_Theta)*sin(Exo_Phi) + cos(0)*cos(Exo_Theta);
        Exo_EDC_vamos = EXOGAM->CorrectionDoppler(Exo_Theta,Exo_Phi,FF_Theta,FF_Phi,FF_Beta13,Exo_E);
      }//FFTheta
      if(ThetaLab!=-1000){
        Exo_EDC_pista = EXOGAM->CorrectionDoppler(Exo_Theta,Exo_Phi,ThetaLab,PhiLab,Beta_pista,Exo_E);
      }//Thetalab
    }//ExoTheta
  } //Size
} //ExogamAnalysis


////////////////////////////////////////////////////////////////////////////////
void Analysis::PistaAnalysis(){  

  //cout << PISTA->EventMultiplicity << endl;
  double Energy = 0;
  if(PISTA->EventMultiplicity==1){
    DeltaE = PISTA->DE[0];
    //Eres = PISTA->E[0];
    Eres = PISTA->back_E[0];
    Energy = DeltaE + Eres;

    strip_DE = PISTA->DE_StripNbr[0];
    strip_E = PISTA->E_StripNbr[0];
    Telescope = PISTA->DetectorNumber[0];
    Time_E = PISTA->back_E_Time[0];
  }

  else if(PISTA->EventMultiplicity==2 && abs(PISTA->DE_StripNbr[0]-PISTA->DE_StripNbr[1])==1){
    DeltaE = PISTA->DE[0] + PISTA->DE[1];
    //Eres = PISTA->E[0];
    Eres = PISTA->back_E[0];
    Energy = DeltaE + Eres;

    Telescope = PISTA->DetectorNumber[0];
    if(PISTA->DE[0]>PISTA->DE[1])
      strip_DE = PISTA->DE_StripNbr[0];
    else
      strip_DE = PISTA->DE_StripNbr[1];

    strip_E = PISTA->E_StripNbr[0];
    Time_E = PISTA->back_E_Time[0];
  }

  if(strip_DE>0 && strip_DE<92 && strip_E>0 && strip_E<58){
    //TVector3 PISTA_pos = PISTA->GetPositionOfInteraction(Telescope, 58-strip_E, strip_DE);
    TVector3 PISTA_pos = PISTA->GetPositionOfInteraction(Telescope, strip_E, strip_DE);

    TVector3 HitDirection = PISTA_pos - PositionOnTarget;
    TVector3 HitDirectionLin = PISTA_pos - PositionOnTargetLin;

    PhiLab = PISTA_pos.Phi();
    Xcalc  = PISTA_pos.X();
    Ycalc  = PISTA_pos.Y();
    Zcalc  = PISTA_pos.Z();

    TVector3 BeamDirection = TVector3(0,0,1);
    BeamDirection.RotateX(m_Beam_ThetaX*3.1415/180);
    BeamDirection.RotateY(m_Beam_ThetaY*3.1415/180);

    ThetaLab = HitDirection.Angle(BeamDirection);
    ThetaLabLin = HitDirectionLin.Angle(BeamDirection);

    ThetaDetectorSurface = HitDirection.Angle(PISTA->GetDetectorNormal(0));

    //double AnglePista;
    //double ThickAlu = 0.5*micrometer;
    //TVector3 NormalPista ;
    //NormalPista = PISTA->GetDetectorNormal(0) ;
    //AnglePista = (HitDirection.Angle(NormalPista));

    DeltaEcorr = DeltaE*cos(ThetaDetectorSurface);
    PID = pow(DeltaEcorr+Eres,1.78)-pow(Eres,1.78);

    ThetaNormalTarget = HitDirection.Angle(TVector3(0,0,1));
    Elab = Energy;

    // 12C case //
    double Elab_12C;
    Elab_12C = geloss_C12Al->Eval(Eres);
    Elab_12C = geloss_C12Al->Eval(Elab_12C);
    Elab_12C += DeltaE;
    Elab_12C = geloss_C12Al->Eval(Elab_12C);
    Elab_12C = geloss_C12C->Eval(Elab_12C);
    C12->SetKineticEnergy(Elab_12C);
    // 10Be case //
    double Elab_10Be;
    Elab_10Be = geloss_Be10Al->Eval(Eres);
    Elab_10Be = geloss_Be10Al->Eval(Elab_10Be);
    Elab_10Be += DeltaE;
    Elab_10Be = geloss_Be10Al->Eval(Elab_10Be);
    Elab_10Be = geloss_Be10C->Eval(Elab_10Be);
    Be10->SetKineticEnergy(Elab_10Be);
    Beta_pista = Be10->GetBeta();

    Ex240Pu = Transfer10Be->ReconstructRelativistic(Elab_10Be, ThetaLab);
    TLorentzVector LV_240Pu = Transfer10Be->LorentzAfterReaction(Elab_10Be, ThetaLab);
    beta_240Pu = LV_240Pu.Beta();
    gamma_240Pu = LV_240Pu.Gamma();
    theta_240Pu = LV_240Pu.Angle(TVector3(0,0,1));
    phi_240Pu = LV_240Pu.Phi();

    TVector3 vector3_FF = TVector3(sin(FF_Theta)*cos(FF_Phi), sin(FF_Theta)*sin(FF_Phi), cos(FF_Theta));
    TVector3 vector3_fiss = TVector3(sin(theta_240Pu)*cos(phi_240Pu), sin(theta_240Pu)*sin(phi_240Pu), cos(theta_240Pu));

    theta_lab_fission = vector3_FF.Angle(vector3_fiss);
    gamma_lab_ff = FF_Gamma13;
    beta_lab_ff  = FF_Beta13;

    gamma_cm_ff = gamma_240Pu*(gamma_lab_ff - beta_240Pu*beta_lab_ff*gamma_lab_ff*cos(theta_lab_fission));
    beta_cm_ff = sqrt(pow(gamma_cm_ff,2)-1)/gamma_cm_ff;
    theta_cm_fission = acos((gamma_240Pu/(beta_cm_ff*gamma_cm_ff))*(beta_lab_ff*gamma_lab_ff*cos(theta_lab_fission) - beta_240Pu*gamma_lab_ff));

    // Other cases
    Ex236U  = Transfer14C->ReconstructRelativistic(Elab, ThetaLab);
    Ex238U  = Elastic->ReconstructRelativistic(Elab_12C, ThetaLab);
    ThetaCM = Transfer10Be->EnergyLabToThetaCM(Elab, ThetaLab)/deg;
    ThetaLab = ThetaLab/deg;
    ThetaLabLin = ThetaLabLin/deg;
    double distance_to_hit = HitDirection.Mag()/1000;
    double C12_tof = C12->GetTimeOfFlight();
    double time_to_hit = distance_to_hit*C12_tof*1e9;
    Pista_Time_Target = Time_E - time_to_hit;
  }

  if(PISTA->EventMultiplicity==4){
    TwoAlphaAnalysis();
  }
}


////////////////////////////////////////////////////////////////////////////////
void Analysis::TwoAlphaAnalysis(){
  m_2alpha = 1;

  double DE1 = PISTA->DE[0];
  double DE2 = PISTA->DE[1];
  double DE3 = PISTA->DE[2];
  double DE4 = PISTA->DE[3];
  double DE = (DE1+DE2+DE3+DE4)/4.;

  double E1 = PISTA->back_E[0];
  double E2 = PISTA->back_E[1];
  double E3 = PISTA->back_E[2];
  double E4 = PISTA->back_E[3];
  double E = (E1+E2+E3+E4)/4.;

  // couple1 // 
  Elab1.push_back(DE1+E1);
  Elab2.push_back(DE3+E2);

  // couple2 //
  Elab1.push_back(DE1+E2);
  Elab2.push_back(DE3+E1);

  int det1 = PISTA->DetectorNumber[0];
  int det2 = PISTA->DetectorNumber[1];
  int strip_DE1 = PISTA->DE_StripNbr[0];
  int strip_DE2 = PISTA->DE_StripNbr[2];

  int strip_E1  = PISTA->E_StripNbr[0];
  int strip_E2  = PISTA->E_StripNbr[1];

  Telescope=det1;

  TVector3 HitDirection;
  if(det1==det2){
    TVector3 PISTA_pos1 = PISTA->GetPositionOfInteraction(det1,strip_E1,strip_DE1);
    TVector3 PISTA_pos2 = PISTA->GetPositionOfInteraction(det1,strip_E1,strip_DE2);
    TVector3 PISTA_pos3 = PISTA->GetPositionOfInteraction(det1,strip_E2,strip_DE1);
    TVector3 PISTA_pos4 = PISTA->GetPositionOfInteraction(det1,strip_E2,strip_DE2);

    TVector3 HitDirection1 = PISTA_pos1 - PositionOnTarget;
    TVector3 HitDirection2 = PISTA_pos2 - PositionOnTarget;
    TVector3 HitDirection3 = PISTA_pos3 - PositionOnTarget;
    TVector3 HitDirection4 = PISTA_pos4 - PositionOnTarget;
    HitDirection = 1./4*(HitDirection1+HitDirection2+HitDirection3+HitDirection4);

    double ThetaLab1 = HitDirection1.Angle(TVector3(0,0,1));
    double ThetaLab2 = HitDirection2.Angle(TVector3(0,0,1));
    double ThetaLab3 = HitDirection3.Angle(TVector3(0,0,1));
    double ThetaLab4 = HitDirection4.Angle(TVector3(0,0,1));

    ThetaLab = (ThetaLab1 + ThetaLab2 + ThetaLab3 + ThetaLab4)/4.;

    /*
       double AnglePista;
       double ThickAlu = 0.5*micrometer;
       TVector3 NormalPista ;
       NormalPista = PISTA->GetDetectorNormal(0) ;
       AnglePista = (HitDirection1.Angle(NormalPista) + HitDirection2.Angle(NormalPista) +HitDirection3.Angle(NormalPista) +HitDirection4.Angle(NormalPista))/4.;


       DeltaE = DE;
    //Eloss
    Eres = E;
    Elab = E;
    Elab = AlphaAl.EvaluateInitialEnergy(Elab, 2*ThickAlu, AnglePista);
    Elab += DE;
    Elab = AlphaAl.EvaluateInitialEnergy(Elab, ThickAlu, AnglePista);
    Elab = AlphaC.EvaluateInitialEnergy(Elab, TargetThickness /2, ThetaLab) ;
    */
  }

  /*else{
    TVector3 PISTA_pos1 = PISTA->GetPositionOfInteraction(det1,strip_E1,strip_DE1);
    TVector3 PISTA_pos2 =Energy = DeltaE + Eres; PISTA->GetPositionOfInteraction(det2,strip_E2,strip_DE2);

    TVector3 HitDirection1 = PISTA_pos1 - PositionOnTarget;
    TVector3 HitDirection2 = PISTA_pos2 - PositionOnTarget;

    double ThetaLab1 = HitDirection1.Angle(TVector3(0,0,1));
    double ThetaLab2 = HitDirection2.Angle(TVector3(0,0,1));

    ThetaLab = (ThetaLab1 + ThetaLab2)/2.;

    }*/

  // *** 8Be construction ** //
  DeltaE = DE;
  ThetaDetectorSurface = HitDirection.Angle(PISTA->GetDetectorNormal(0));
  DeltaEcorr = DeltaE*cos(ThetaDetectorSurface);
  Eres = E;
  Elab = DE + E;
  Ex242Pu = Transfer8Be->ReconstructRelativistic(Elab, ThetaLab);
  ThetaCM = Transfer8Be->EnergyLabToThetaCM(Elab, ThetaLab)/deg;
  ThetaLab = ThetaLab/deg;

}

/////////////////////////////////////////////////////////////////////////////////
void Analysis::VamosAnalysis(){
    if(abs(FPMW->XtLin)<5 && abs(FPMW->YtLin)<5){
        //XTarget = FPMW->Xt*cos(20*deg+FPMW->Theta_in)/cos(FPMW->Theta_in);
        XTarget = FPMW->Xt*(cos(FPMW->Theta_in)/cos(20*deg+FPMW->Theta_in));
        YTarget = FPMW->Yt;

        XTargetLin = FPMW->XtLin*(cos(FPMW->Theta_in_Lin)/cos(20*deg+FPMW->Theta_in_Lin));
        YTargetLin = FPMW->YtLin;
        ZTarget = 0;

        FF_Theta = FPMW->Theta_in+m_Vamos_Angle*deg;
        FF_Phi = FPMW->Phi_in;

        Xmean_iter += XTarget;
        Ymean_iter += YTarget;
        Xmean_iterLin += XTargetLin;
        Ymean_iterLin += YTargetLin;

        iteration++;
        if(iteration%100==0){
            Xmean = Xmean_iter/iteration;
            Ymean = Ymean_iter/iteration;

            XmeanLin = Xmean_iterLin/iteration;
            YmeanLin = Ymean_iterLin/iteration;

            iteration = 0;
            Xmean_iter = 0;
            Ymean_iter = 0;


            Xmean_iterLin = 0;
            Ymean_iterLin = 0;
        }
    } //If some linearisation is done 
    else{
        XTargetLin = XmeanLin;
        YTargetLin = YmeanLin;
        XTarget = Xmean;
        YTarget = Ymean;
        ZTarget = 0;
    }

    XTarget += m_XTarget_offset;
    YTarget += m_YTarget_offset;
    ZTarget += m_ZTarget_offset;

    XTargetLin += m_XTarget_offset;
    YTargetLin += m_YTarget_offset +0.2;

    //cout << XTarget << endl;


    PositionOnTarget = TVector3(XTarget,YTarget,ZTarget);
    PositionOnTargetLin = TVector3(XTargetLin,YTargetLin,ZTarget);

    if( MTOF_FP0_T0VM == 1 && MTOF_FP0_T1VM ==1 ){


        UShort_t FPMWPat = MTOF_FP0_T0VN[0]; //20 section
        FPMW_Section = FPMWPat;
        IC->SetFPMWSection(FPMW_Section);
        IC->BuildSimplePhysicalEvent();
        double Theta = -1000;


        if(FPMW->Xf!=-1000){

            FF_DE      = IC->DE;
            FF_DE_IC14   = 0.5*(IC->fIC_raw[1] +  IC->fIC_raw[2] + IC->fIC_raw[3]) + IC->fIC_raw[4] ;
            FF_Eres = IC->Eres;

            Tracking->CalculateReconstruction(FPMW->Xf, 1000*FPMW->Thetaf, m_Brho_ref, FF_Brho, Theta, FF_Path);
            // FF_Path is in cm !
            //
            FF_Y1  = FPMW->PositionY[0];
            FF_Y3  = FPMW->PositionY[2];

            //FF_IC_Y = FPMW->Yf + (1442.6+6774.4-7600)*tan(FPMW->Phif/1000);
            FF_IC_Y = FPMW->Yf + (1442.6+6774.4-7600)*tan(FPMW->Phif);
            FF_IC_X = FPMW->Xf + (1442.6+6774.4-7600)*tan(FPMW->Thetaf);

            // T13 //
            double path1 = FPMW->GetDetectorPositionZ(0)/10./cos(FPMW->Theta_in)/cos(FPMW->Phi_in);
            double path2 = (FPMW->GetDetectorPositionZ(2)-7600)/10./cos(FPMW->Thetaf);

            //Toff
            vector<double> Toff13 , Toff14, Toff23, Toff24;
            const char* Path13 = "macro/mwpc/Toff/output/Toff13.txt";
            const char* Path14 = "macro/mwpc/Toff/output/Toff14.txt";
            const char* Path23 = "macro/mwpc/Toff/output/Toff23.txt";
            const char* Path24 = "macro/mwpc/Toff/output/Toff24.txt";

            Toff13 = TxtToVector(Path13);
            Toff14 = TxtToVector(Path14);
            Toff23 = TxtToVector(Path23);
            Toff24 = TxtToVector(Path24);

            double Toff[20] = {0, 588.0, 588.5, 587.95, 588.04, 587.72, 587.92, 587.9, 587.9, 588.66, 588.80, 588.67, 588.64, 588.75, 588.47, 588.65, 588.65, 588.67, 589.05, 590.3};
            // To know time of electron drift in a section of the chio
            for (int seg = 0 ; seg < IC->fIC_TS.size() ; seg++){
                    FF_DriftTime.push_back(10* (IC->fIC_TS.at(seg) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))));
            }



            FF_T13 = - MTOF_FP0_T0V[0] + Toff[FPMWPat];
            //FF_T13 = -Time->GetTime_MWPC13(0) - Toff13[FPMW_Section-1] + 589;
            FF_D13 = FF_Path - path1 + path2; 
            FF_V13 = FF_D13/FF_T13;
            FF_Beta13 = FF_V13/29.9792458;
            FF_Gamma13 = 1./sqrt(1.0-FF_Beta13*FF_Beta13);


            FF_AoQ13 = (FF_Brho/(3.107*FF_Beta13*FF_Gamma13));
            FF_M13 = IC->EtotInit/(931.5016*(FF_Gamma13-1.));
            FF_Q13 = FF_M13/FF_AoQ13;

            FF_Mass13 = int(FF_Q13+0.5)*FF_AoQ13;

            //FF_DriftTime[FPMW_Section] = 10*(fVAMOS_TS_sec[FPMW_Section]- MT)
        }
    }
    else{
        FPMW_Section = -1;
    }

}

vector<double> Analysis::TxtToVector(const char *Path){
    string line;
    vector<double> values;
    ifstream file(Path); 

    if (file.is_open()) {
        while (std::getline(file, line)) {
            try {
                values.push_back(std::stod(line));
            } catch (const std::invalid_argument& e) {
                std::cerr << "Invalid number in line: " << line << '\n';
            }
        }
        file.close();
    } else {
        std::cerr << "Error opening file.\n";
    }

    return values;
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::InitOutputBranch(){
    RootOutput::getInstance()->GetTree()->Branch("Time",&Time);

    RootOutput::getInstance()->GetTree()->Branch("BeamEnergy",&BeamEnergy,"BeamEnergy/D");

    RootOutput::getInstance()->GetTree()->Branch("XTarget",&XTarget,"XTarget/D");
    RootOutput::getInstance()->GetTree()->Branch("XTargetLin",&XTargetLin,"XTargetLin/D");
    RootOutput::getInstance()->GetTree()->Branch("YTarget",&YTarget,"YTarget/D");
    RootOutput::getInstance()->GetTree()->Branch("YTargetLin",&YTargetLin,"YTargetLin/D");
    RootOutput::getInstance()->GetTree()->Branch("ZTarget",&ZTarget,"ZTarget/D");

    RootOutput::getInstance()->GetTree()->Branch("Ex242Pu",&Ex242Pu,"Ex242Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("Ex240Pu",&Ex240Pu,"Ex240Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("Ex240PuLin",&Ex240PuLin,"Ex240PuLin/D");
    RootOutput::getInstance()->GetTree()->Branch("Ex236U",&Ex236U,"Ex236U/D");
    RootOutput::getInstance()->GetTree()->Branch("Ex238U",&Ex238U,"Ex238U/D");

    RootOutput::getInstance()->GetTree()->Branch("DeltaE",&DeltaE,"DeltaE/D");
    RootOutput::getInstance()->GetTree()->Branch("DeltaEcorr",&DeltaEcorr,"DeltaEcorr/D");
    RootOutput::getInstance()->GetTree()->Branch("Eres",&Eres,"Eres/D");

    RootOutput::getInstance()->GetTree()->Branch("Telescope",&Telescope,"Telescope/I");
    RootOutput::getInstance()->GetTree()->Branch("PID",&PID,"PID/D");

    RootOutput::getInstance()->GetTree()->Branch("Elab",&Elab,"Elab/D");
    RootOutput::getInstance()->GetTree()->Branch("ThetaLab",&ThetaLab,"ThetaLab/D");

    RootOutput::getInstance()->GetTree()->Branch("Beta_pista",&Beta_pista,"Beta_pista/D");
    RootOutput::getInstance()->GetTree()->Branch("ThetaLabLin",&ThetaLabLin,"ThetaLabLin/D");
    RootOutput::getInstance()->GetTree()->Branch("ThetaDetectorSurface",&ThetaDetectorSurface,"ThetaDetectorSurface/D");
    RootOutput::getInstance()->GetTree()->Branch("PhiLab",&PhiLab,"PhiLab/D");
    RootOutput::getInstance()->GetTree()->Branch("ThetaCM",&ThetaCM,"ThetaCM/D");

    RootOutput::getInstance()->GetTree()->Branch("Xcalc",&Xcalc,"Xcalc/D");
    RootOutput::getInstance()->GetTree()->Branch("Ycalc",&Ycalc,"Ycalc/D");
    RootOutput::getInstance()->GetTree()->Branch("Zcalc",&Zcalc,"Zcalc/D");

    RootOutput::getInstance()->GetTree()->Branch("strip_DE",&strip_DE,"strip_DE/I");
    RootOutput::getInstance()->GetTree()->Branch("strip_E",&strip_E,"strip_E/I");
    RootOutput::getInstance()->GetTree()->Branch("Time_E",&Time_E,"Time_E/D");

    RootOutput::getInstance()->GetTree()->Branch("Pista_Time_Target",&Pista_Time_Target,"Pista_Time_Target/D");
    RootOutput::getInstance()->GetTree()->Branch("Vamos_Time_Target",&Vamos_Time_Target,"Vamos_Time_Target/D");

    RootOutput::getInstance()->GetTree()->Branch("FF_DE",&FF_DE,"FF_DE/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_DE_IC14",&FF_DE_IC14,"FF_DE_IC14/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Eres",&FF_Eres,"FF_Eres/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Z",&FF_Z,"FF_Z/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Theta",&FF_Theta,"FF_Theta/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Phi",&FF_Phi,"FF_Phi/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Brho",&FF_Brho,"FF_Brho/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Path",&FF_Path,"FF_Path/D");

    RootOutput::getInstance()->GetTree()->Branch("FF_DriftTime",&FF_DriftTime);
    RootOutput::getInstance()->GetTree()->Branch("FF_Y1",&FF_Y1,"FF_Y1/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Y3",&FF_Y3,"FF_Y3/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_IC_Y",&FF_IC_Y,"FF_IC_Y/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_IC_X",&FF_IC_X,"FF_IC_X/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_D13",&FF_D13,"FF_D13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_T13",&FF_T13,"FF_T13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_V13",&FF_V13,"FF_V13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_AoQ13",&FF_AoQ13,"FF_AoQ13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Beta13",&FF_Beta13,"FF_Beta13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Gamma13",&FF_Gamma13,"FF_Gamma13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Q13",&FF_Q13,"FF_Q13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_M13",&FF_M13,"FF_M13/D");
    RootOutput::getInstance()->GetTree()->Branch("FF_Mass13",&FF_Mass13,"FF_Mass13/D");

    RootOutput::getInstance()->GetTree()->Branch("FPMW_Section",&FPMW_Section,"FPMW_Section/I");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP0_T0VM",&MTOF_FP0_T0VM,"MTOF_FP0_T0VM/I");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP0_T1VM",&MTOF_FP0_T1VM,"MTOF_FP0_T1VM/I");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP1_T0VM",&MTOF_FP1_T0VM,"MTOF_FP1_T0VM/I");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP1_T1VM",&MTOF_FP1_T1VM,"MTOF_FP1_T1VM/I");

    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP0_T0VN",&MTOF_FP0_T0VN,"MTOF_FP0_T0VN/S");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP0_T1VN",&MTOF_FP0_T1VN,"MTOF_FP0_T1VN/S");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP1_T0VN",&MTOF_FP1_T0VN,"MTOF_FP1_T0VN/S");
    RootOutput::getInstance()->GetTree()->Branch("MTOF_FP1_T1VN",&MTOF_FP1_T1VN,"MTOF_FP1_T1VN/S");

    RootOutput::getInstance()->GetTree()->Branch("theta_lab_fission",&theta_lab_fission,"theta_lab_fission/D");
    RootOutput::getInstance()->GetTree()->Branch("theta_cm_fission",&theta_cm_fission,"theta_cm_fission/D");
    RootOutput::getInstance()->GetTree()->Branch("theta_240Pu",&theta_240Pu,"theta_240Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("phi_240Pu",&phi_240Pu,"phi_240Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("beta_240Pu",&beta_240Pu,"beta_240Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("gamma_240Pu",&gamma_240Pu,"gamma_240Pu/D");
    RootOutput::getInstance()->GetTree()->Branch("gamma_lab_ff",&gamma_lab_ff,"gamma_lab_ff/D");
    RootOutput::getInstance()->GetTree()->Branch("gamma_cm_ff",&gamma_cm_ff,"gamma_cm_ff/D");
    RootOutput::getInstance()->GetTree()->Branch("beta_lab_ff",&beta_lab_ff,"beta_lab_ff/D");
    RootOutput::getInstance()->GetTree()->Branch("beta_cm_ff",&beta_cm_ff,"beta_cm_ff/D");

    RootOutput::getInstance()->GetTree()->Branch("Elab1",&Elab1);
    RootOutput::getInstance()->GetTree()->Branch("Elab2",&Elab2);
    RootOutput::getInstance()->GetTree()->Branch("m_2alpha",&m_2alpha,"m_2alpha/I");

    RootOutput::getInstance()->GetTree()->Branch("Exo_cosa",&Exo_cosa,"Exo_cosa/D");
    RootOutput::getInstance()->GetTree()->Branch("Exo_E",&Exo_E,"Exo_E/D");
    RootOutput::getInstance()->GetTree()->Branch("Exo_EDC_vamos",&Exo_EDC_vamos,"Exo_EDC_vamos/D");
    RootOutput::getInstance()->GetTree()->Branch("Exo_EDC_pista",&Exo_EDC_pista,"Exo_EDC_pista/D");
    RootOutput::getInstance()->GetTree()->Branch("Exo_Theta",&Exo_Theta,"Exo_Theta/D");
    RootOutput::getInstance()->GetTree()->Branch("Exo_Phi",&Exo_Phi,"Exo_Phi/D");

    RootOutput::getInstance()->GetTree()->Branch("VAMOS_TS_hour",&VAMOS_TS_hour,"VAMOS_TS_hour/D");
    RootOutput::getInstance()->GetTree()->Branch("PISTA_TS_hour",&PISTA_TS_hour,"PISTA_TS_hour/D");



}

////////////////////////////////////////////////////////////////////////////////
void Analysis::InitInputBranch(){
    RootInput::getInstance()->GetChain()->SetBranchStatus("Time",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("Time",&Time);

    RootInput::getInstance()->GetChain()->SetBranchStatus("T_TMW0_FPMW0",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("T_TMW0_FPMW0",&T13);

    RootInput::getInstance()->GetChain()->SetBranchStatus("T_TMW0_FPMW1",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("T_TMW0_FPMW1",&T14);

    RootInput::getInstance()->GetChain()->SetBranchStatus("T_TMW1_FPMW0",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("T_TMW1_FPMW0",&T23);

    RootInput::getInstance()->GetChain()->SetBranchStatus("T_TMW1_FPMW1",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("T_TMW1_FPMW1",&T24);

    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T0VN",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T0VN",&MTOF_FP0_T0VN);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T1VN",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T1VN",&MTOF_FP0_T1VN);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T0VN",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T0VN",&MTOF_FP1_T0VN);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T1VN",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T1VN",&MTOF_FP1_T1VN);

    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T0V",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T0V",&MTOF_FP0_T0V);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T1V",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T1V",&MTOF_FP0_T1V);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T0V",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T0V",&MTOF_FP1_T0V);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T1V",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T1V",&MTOF_FP1_T1V);


    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T0VM",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T0VM",&MTOF_FP0_T0VM);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP0_T1VM",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP0_T1VM",&MTOF_FP0_T1VM);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T0VM",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T0VM",&MTOF_FP1_T0VM);
    RootInput::getInstance()->GetChain()->SetBranchStatus("MTOF_FP1_T1VM",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("MTOF_FP1_T1VM",&MTOF_FP1_T1VM);

    RootInput::getInstance()->GetChain()->SetBranchStatus("fVAMOS_TS_sec",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("fVAMOS_TS_sec",&fVAMOS_TS_sec);
    RootInput::getInstance()->GetChain()->SetBranchStatus("fPISTA_TS_sec",true);
    RootInput::getInstance()->GetChain()->SetBranchAddress("fPISTA_TS_sec",&fPISTA_TS_sec);
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::ReInitValue(){
    BeamEnergy = -1000;
    Ex242Pu = -1000;
    Ex240Pu = -1000;
    Ex240PuLin = -1000;
    Ex236U = -1000;
    Ex238U = -1000;
    DeltaE = -1000;
    DeltaEcorr = -1000;
    Eres = -1000;
    Elab = -1000;
    ThetaLab = -1000;
    Beta_pista = -1;
    ThetaDetectorSurface = -1000;
    PhiLab = -1000;
    ThetaCM = -1000;
    XTarget = -1000;
    XTargetLin = -1000;
    YTarget = -1000;
    YTargetLin = -1000;
    ZTarget = -1000;
    Xcalc = -1000;
    Ycalc = -1000;
    Zcalc = -1000;
    PID = -1000;
    Telescope = -1;
    strip_DE = -1;
    strip_E = -1;
    Time_E = -1000;
    Pista_Time_Target = -1000;
    Vamos_Time_Target = -1000;

    Exo_cosa = -100;
    Exo_E = -100;
    Exo_EDC_vamos = -100;
    Exo_EDC_pista = -100;
    Exo_Theta = -100;
    Exo_Phi = -100;

    FF_Theta = -100;
    FF_Phi = -100;
    FF_Brho = -1;
    FF_Path = -1;
    FF_Y1 = -1000;
    FF_Y3 = -1000;
    FF_IC_X = -1000;
    FF_IC_Y = -1000;
    FF_D13 = -1;
    FF_T13 = -1;
    FF_V13 = -1;
    FF_AoQ13 = -1;
    FF_Beta13 = -1;
    FF_Gamma13 = -1;
    FF_Q13 = -1;
    FF_M13 = -1;
    FF_Mass13 = -1;
    FPMW_Section = -1;
    FF_DriftTime.clear();

    theta_lab_fission = -100;
    theta_cm_fission = -100;
    theta_240Pu = -100;
    phi_240Pu = -100;
    beta_240Pu = -100;
    gamma_240Pu = -100;
    gamma_lab_ff = -100;
    gamma_cm_ff = -100;
    beta_lab_ff = -100;
    beta_cm_ff = -100;


    m_2alpha = 0;
    Elab1.clear();
    Elab2.clear();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::ReadAnalysisConfig(){
    bool ReadingStatus = false;

    string filename = "AnalysisConfig.dat";

    // open analysis config file
    ifstream AnalysisConfigFile;
    AnalysisConfigFile.open(filename.c_str());

    if (!AnalysisConfigFile.is_open()) {
        cout << " No AnalysisConfig.dat found: Default parameter loaded for Analayis " << filename << endl;
        return;
    }
    cout << "**** Loading user parameter for Analysis from AnalysisConfig.dat " << endl;

    // Save it in a TAsciiFile
    TAsciiFile* asciiConfig = RootOutput::getInstance()->GetAsciiFileAnalysisConfig();
    asciiConfig->AppendLine("%%% AnalysisConfig.dat %%%");
    asciiConfig->Append(filename.c_str());
    asciiConfig->AppendLine("");
    // read analysis config file
    string LineBuffer,DataBuffer,whatToDo;
    while (!AnalysisConfigFile.eof()) {
        // Pick-up next line
        getline(AnalysisConfigFile, LineBuffer);

        // search for "header"
        string name = "AnalysisConfig";
        if (LineBuffer.compare(0, name.length(), name) == 0) 
            ReadingStatus = true;

        // loop on tokens and data
        while (ReadingStatus ) {
            whatToDo="";
            AnalysisConfigFile >> whatToDo;

            // Search for comment symbol (%)
            if (whatToDo.compare(0, 1, "%") == 0) {
                AnalysisConfigFile.ignore(numeric_limits<streamsize>::max(), '\n' );
            }

            else if (whatToDo=="XTARGET_OFFSET") {
                AnalysisConfigFile >> DataBuffer;
                m_XTarget_offset = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_XTarget_offset << endl;
            }
            else if (whatToDo=="YTARGET_OFFSET") {
                AnalysisConfigFile >> DataBuffer;
                m_YTarget_offset = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_YTarget_offset << endl;
            }
            else if (whatToDo=="ZTARGET_OFFSET") {
                AnalysisConfigFile >> DataBuffer;
                m_ZTarget_offset = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_ZTarget_offset << endl;
            }
            else if (whatToDo=="BEAM_THETAX") {
                AnalysisConfigFile >> DataBuffer;
                m_Beam_ThetaX = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_Beam_ThetaX << endl;
            }
            else if (whatToDo=="BEAM_THETAY") {
                AnalysisConfigFile >> DataBuffer;
                m_Beam_ThetaY = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_Beam_ThetaY << endl;
            }
            else if (whatToDo=="BRHO_REF") {
                AnalysisConfigFile >> DataBuffer;
                m_Brho_ref = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_Brho_ref << endl;
            }
            else if (whatToDo=="VAMOS_ANGLE") {
                AnalysisConfigFile >> DataBuffer;
                m_Vamos_Angle = atof(DataBuffer.c_str());
                cout << "**** " << whatToDo << " " << m_Vamos_Angle << endl;
            }

            else {
                ReadingStatus = false;
            }
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::End(){
}


////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the DetectorFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VAnalysis* Analysis::Construct(){
    return (NPL::VAnalysis*) new Analysis();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
    class proxy{
        public:
            proxy(){
                NPL::AnalysisFactory::getInstance()->SetConstructor(Analysis::Construct);
            }
    };

    proxy p;
}

