#include <TChain.h>
#include <TFile.h>
#include <TH1.h>
#include <vector>

using namespace std ;

double TransfoPos(double X, vector<double> &EdgeBin,
                  vector<double> &NewEdgeBin);

void readVectorsFromFile(vector<vector<vector<double>>>& vec1, vector<vector<vector<double>>>& vec2, const string& filename) ;

void LinearisationTransfo(const char*InputFile) {
  // **************************************************************************
  // This macro linearize the reconstructed position for each event on each FPMW
  //
  // It will load a YAML file containing the old and new bin edges of each FPMW
  // 
  // Then it will check if the event is stored in the range of a old bin and
  // transform it in the corresponding new bin.
  //
  // Input :
  //    Run.root : Run to be treated
  //    
  //    Linearisation.yaml : yaml file containing the old and new bin edges for
  //    each of the detector.
  //
  //
  // Output:
  //    RunLinearised.root : Same as run but with a new branch of linearised
  //    position
  // **************************************************************************

  //========================================================================
  //-------------------Open tree and define Hist----------------------------
  //========================================================================

  // Import the chain
  TChain *ch = new TChain("PhysicsTree");
  //for (int run = RunStart; run <= RunStop; run++) {
  ch->Add(Form("../../root/analysis/%s.root", InputFile));
  //} // End of run loop

  // HERE PUT THE RIGHT BIN EDGES !!!! to automatize
  vector<TH1F *> VarBin, DetUnmod, DetLinea;
  for (int i = 1; i <= 4; i++) {
    std::string DetUnmodXName = "hDetXUnmod" + std::to_string(i);
    std::string DetLineaXName = "hDetXLinea" + std::to_string(i);
    std::string DetUnmodYName = "hDetYUnmod" + std::to_string(i);
    std::string DetLineaYName = "hDetYLinea" + std::to_string(i);

    TH1F *DetUnmodXTemp;
    TH1F *DetLineaXTemp;
    TH1F *DetUnmodYTemp;
    TH1F *DetLineaYTemp;

    if (i == 1) {
      DetUnmodXTemp =
          new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(), 400, -40, 40);
      DetLineaXTemp =
          new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(), 400, -40, 40);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 600, -40, 40);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 600, -40, 40);
    } // End if det 1

    if (i == 2) {
      DetUnmodXTemp =
          new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(), 630, -33, 33);
      DetLineaXTemp =
          new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(), 630, -33, 33);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 920, -46, 46);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 920, -46, 46);

    } // End if det 2

    if (i == 3 || i == 4) {
      DetUnmodXTemp = new TH1F(DetUnmodXName.c_str(), DetUnmodXName.c_str(),
                               10000, -500, 500);
      DetLineaXTemp = new TH1F(DetLineaXName.c_str(), DetLineaXName.c_str(),
                               10000, -500, 500);
      DetUnmodYTemp =
          new TH1F(DetUnmodYName.c_str(), DetUnmodYName.c_str(), 1600, -80, 80);
      DetLineaYTemp =
          new TH1F(DetLineaYName.c_str(), DetLineaYName.c_str(), 1600, -80, 80);
    } // End if det 3 or 4
    DetUnmod.push_back(DetUnmodXTemp);
    DetUnmod.push_back(DetUnmodYTemp);
    DetLinea.push_back(DetLineaXTemp);
    DetLinea.push_back(DetLineaYTemp);
  } // End of i loop
  
  //========================================================================
  //-----------------------Importing conditional var------------------------
  //========================================================================
  TFPMWPhysics *FPMW = new TFPMWPhysics();
  ch->SetBranchAddress("FPMW", &FPMW);

  // ************************************************************************
  //                                Load YAML  
  // ************************************************************************
  vector<vector<vector<double>>> EdgeBinDet, NewEdgeBinDet;
  //  LoadFromYaml(EdgeBinDet, NewEdgeBinDet, "Linearisation.yaml");
  readVectorsFromFile(EdgeBinDet,NewEdgeBinDet,"Output/Linearisation.txt");
  for (int det = 0 ; det<NewEdgeBinDet.size() ; det++){
    vector <double> VarBinHolder;
    for(int mm = 0 ; mm< NewEdgeBinDet[det].size() ; mm ++){
      for (int elem =0 ; elem < NewEdgeBinDet[det][mm].size()-1     ; elem++){
        VarBinHolder.push_back(NewEdgeBinDet[det][mm][elem]);
      }
    }
    std :: string VarBinName = "hVarbin" + std::to_string(det);
    VarBin.push_back(new TH1F(VarBinName.c_str(),VarBinName.c_str(),VarBinHolder.size()-1, & VarBinHolder[0]));     
  }
  bool Verbose = false;
  //========================================================================
  //---------------------------Treat Event----------------------------------
  //========================================================================
  int Nentries = ch->GetEntries();
  int GoodEntries = 0;

  for (int entries = 0; entries < Nentries; entries++) {
    ch->GetEntry(entries);
    if (entries % 100000 == 0) {
      cout << "\033[92m\r Processing tree..."
           << (double)entries / Nentries * 100 << "\% done" << flush;
    } // End cout loading

    // Verification that we are not on a empty event for the FPMW
    if (FPMW->DetectorNbr.size() > 0) {
      GoodEntries += 1;

      // Parsing through the different FPMW
      for (int DetectorNbr = 0; DetectorNbr < FPMW->DetectorNbr.size();
           DetectorNbr++) {

        double X = FPMW->PositionX[DetectorNbr];
        double Y = FPMW->PositionY[DetectorNbr];

        // ***************************************************************
        //                      Linearise X detector
        // ***************************************************************

        int IndexX = DetectorNbr * 2;
        for (int binX = 0; binX < EdgeBinDet[IndexX].size() - 1; binX++) {
          // If X is in the correct range
          if (EdgeBinDet[IndexX][binX].size() != 0) {
            if (X >= EdgeBinDet[IndexX][binX][0] &&
                X < EdgeBinDet[IndexX][binX]
                [EdgeBinDet[IndexX][binX].size() - 1]) {
              // Apply the coorect transformation to X
              double XNew = TransfoPos(X, EdgeBinDet[IndexX][binX],
                  NewEdgeBinDet[IndexX][binX]);
              // Fill of histogram for testing
              VarBin[IndexX]->Fill(XNew);
              DetUnmod[IndexX]->Fill(X);
              DetLinea[IndexX]->Fill(XNew);

              // Some cout
              if (Verbose == true) {
                cout << X << " " << XNew << "  " << endl;
                cout << "************************" << endl;
              }
            } // end check in range
          }   // end check empty
        }     // end loop mmX

        // ***************************************************************
        //                      Linearise Y detector
        // ***************************************************************


        int IndexY = DetectorNbr * 2 + 1;
        for (int binY = 0; binY < EdgeBinDet[IndexY].size() - 1;
            binY++) {
          if (EdgeBinDet[IndexY][binY].size() != 0) {
            if (Y >= EdgeBinDet[IndexY][binY][0] &&
                Y < EdgeBinDet[IndexY][binY]
                [EdgeBinDet[IndexY][binY].size() - 1]) {
              double YNew = TransfoPos(Y, EdgeBinDet[IndexY][binY],
                  NewEdgeBinDet[IndexY][binY]);

              VarBin[IndexY]->Fill(YNew);
              DetUnmod[IndexY]->Fill(Y);
              DetLinea[IndexY]->Fill(YNew);
              if (Verbose == true) {
                cout << Y << " " << YNew << "  " << endl;
                cout << "************************" << endl;
              }
            } // end check in range
          }   // End non empty
        }     // End loop BINY
      }       // Loop detnbr
    }         // Non empty event
  }           // End loop on entries

  cout << endl << "Good entries" << GoodEntries << endl;

  //========================================================================
  //---------------------Output histogram-----------------------------------
  //========================================================================
  TFile *OutputHist = new TFile("AfterLinea.root", "RECREATE");
  std::vector<TCanvas *> canvas, cLinea;
  for (int i = 0; i < DetUnmod.size(); i++) {
    std::string CanvasNameLinea = "Linearised" + std::to_string(i);
    TCanvas *Clin = new TCanvas(CanvasNameLinea.c_str(),
        CanvasNameLinea.c_str(), 2500, 400);
    Clin->Divide(3, 1);
    cLinea.push_back(Clin);

    cLinea[i]->cd(1);
    DetUnmod[i]->SetXTitle("Reconstructed X");
    DetUnmod[i]->SetYTitle("Number of events");
    DetUnmod[i]->Draw();
    DetUnmod[i]->Write();

    cLinea[i]->cd(2);
    DetLinea[i]->SetXTitle("Reconstructed X");
    DetLinea[i]->SetYTitle("Number of events");
    DetLinea[i]->Draw("");
    DetLinea[i]->Write();

    cLinea[i]->cd(3);
    VarBin[i]->SetXTitle("Reconstructed X");
    VarBin[i]->SetYTitle("Number of events");
    VarBin[i]->Draw("");
    VarBin[i]->Write();
    gStyle->SetOptStat(1111);
  }
  OutputHist->Close();
} // end macro

//========================================================================
//--------------------------TransfoX--------------------------------------
//========================================================================

double TransfoPos(double X, vector<double> &EdgeBin,
    vector<double> &NewEdgeBin) {
  // ************************************************************************
  // Transform an inputed reconstructed position in order to linearise the
  // distribution of event in a FPMW.
  //
  // Input :
  //    EdgeBin : The range of the bin of the FPMW to treat, this vector is
  //    generated by LinearisationYaml.cxx
  //
  //    NewEdgeBin : The corresponding vector of new bin associated to each old
  //    bin of the FPMW. Those bins have a size made to linearise the
  //    distribution. They are also generated by LinearisationYaml.cxx
  //
  //    X : The reconstructed position before linearisation
  // Output :
  //    Xnew : The reconstructed position after linearisation.
  // ************************************************************************
  double XNew = -1000;
  bool TransfoDone = false;
  for (int i = 0; i < EdgeBin.size() - 1; i++) {
    if (X >= EdgeBin[i] && X < EdgeBin[i + 1]) {
      if (TransfoDone == 1 ) {
        continue;
      }
      double Xold = X;
      double Scale2 = (X - EdgeBin[i]) / (EdgeBin[i + 1] - EdgeBin[i]);
      XNew = Scale2 * (NewEdgeBin[i + 1] - NewEdgeBin[i]) + NewEdgeBin[i];
      TransfoDone = true;
      // Sanity check print
      if ((XNew < NewEdgeBin[i] || XNew > NewEdgeBin[i + 1]) && false) {
        cout << " FUNCTION  " << endl;
        cout << i << endl;
        cout << "MinX " << EdgeBin[i] << " MaxX " << EdgeBin[i + 1] << endl;
        cout << "Xold " << Xold << endl;
        cout << "NewMinX " << NewEdgeBin[i] << " NewMaxX " << NewEdgeBin[i + 1]
          << endl;
        cout << "Xnew  " << XNew << endl;
        cout << endl << endl;
      }
    }
  }
  return XNew;
} // end transfo x

void readVectorsFromFile(vector<vector<vector<double>>>& vec1, vector<vector<vector<double>>>& vec2, const string& filename) {
    ifstream inFile(filename);
    if (!inFile.is_open()) {
        cerr << "Unable to open file for reading: " << filename << endl;
        return;
    }

    vec1.clear();
    vec2.clear();
    vector<vector<vector<double>>>* currentVec = &vec1;
    string line;
    vector<vector<double>> det;
    vector<double> bin;

    while (getline(inFile, line)) {
        if (line.find("Vector 2:") != string::npos) {
            if (!bin.empty()) {
                det.push_back(bin);
                bin.clear();
            }
            if (!det.empty()) {
                vec1.push_back(det);
                det.clear();
            }
            currentVec = &vec2;
            continue;
        }

        if (line.find("Det") != string::npos) {
            if (!bin.empty()) {
                det.push_back(bin);
                bin.clear();
            }
            if (!det.empty()) {
                currentVec->push_back(det);
                det.clear();
            }
            continue;
        }

        if (line.find("Bin") != string::npos) {
            if (!bin.empty()) {
                det.push_back(bin);
                bin.clear();
            }
            istringstream iss(line.substr(line.find(":") + 1));
            double value;
            while (iss >> value) {
                bin.push_back(value);
            }
            det.push_back(bin);
            bin.clear();
            continue;
        }

        if (line.empty()) {
            if (!bin.empty()) {
                det.push_back(bin);
                bin.clear();
            }
            if (!det.empty()) {
                currentVec->push_back(det);
                det.clear();
            }
        }
    }

    if (!bin.empty()) {
        det.push_back(bin);
    }
    if (!det.empty()) {
        currentVec->push_back(det);
    }

    inFile.close();
}

