#ifndef Analysis_h 
#define Analysis_h
/*****************************************************************************
 * Copyright (C) 2009-2014    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: a.matta@surrey.ac.uk      *
 * Edited for e793s                                                          *
 *                                                                           *
 * Creation Date  :                                                          *
 * Last update    : May 2021                                                 *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 * Class describing the property of an Analysis object                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#include"NPVAnalysis.h"
#include"NPEnergyLoss.h"
#include"NPReaction.h"
#include"NPBeam.h"
#include"RootOutput.h"
#include"RootInput.h"
#include "TF1.h"
#include "TInitialConditions.h"
#include "TReactionConditions.h"
#include "TInteractionCoordinates.h"
#include "TMust2Physics.h"
#include "TMugastPhysics.h"
#include "TCATSPhysics.h"
//#include "TPlasticPhysics.h"
//#include "../../NPLib/Detectors/ModularLeaf/TModularLeafPhysics.h"
#include "TModularLeafPhysics.h"
#include <TRandom3.h>
#include <TVector3.h>
#include <TMath.h>
#include <bitset>

int NumThetaAngleBins = 1800;// 180 = 1 deg, 360 = 0.5 deg, 900 = 0.2 deg, 1800 = 0.1 deg

auto ThetaCM_emmitted = new TH1F("ThetaCM_emmitted","ThetaCM_emmitted",NumThetaAngleBins,0,180);
auto ThetaCM_detected_MG = new TH1F("ThetaCM_detected_MG","ThetaCM_detected_MG",NumThetaAngleBins,0,180);

auto ThetaLab_emmitted = new TH1F("ThetaLab_emmitted","ThetaLab_emmitted",NumThetaAngleBins,0,180);
auto ThetaLab_detected_MG = new TH1F("ThetaLab_detected_MG","ThetaLab_detected_MG",NumThetaAngleBins,0,180);

double degtorad = M_PI/180.;
vector<double> clineVal, clineX;
bool filledCline;

TH1F *ThetaCM_detected_MGX[7];
TH1F *ThetaLab_detected_MGX[7];

class Analysis: public NPL::VAnalysis{
  public:
    Analysis();
    ~Analysis();

  public: 
    void Init();
    void TreatEvent();
    void End();

    void InitOutputBranch();
    void InitInputBranch();
    void SetBranchStatus();
    void CalculateVamos();
    void ReInitValue();
    static NPL::VAnalysis* Construct();

  private:
    bool runningAuTarget;
    std::vector<bool> hasAuTarget;

    unsigned int ParticleMult;
    unsigned int GammaMult;
    //double EDC;
    std::vector<double> EDC;
    vector<double> RawGammaEnergy;
    vector<double> BetaUsedByAddBackEDC;
    vector<double> AddBack_EDC;
    vector<double> AddBack_EDC_Event1;
    vector<double> AddBack_EDC_Event2;
    vector<double> AGATA_Theta;
    vector<double> AGATA_Phi;
    vector<double> AGATA_GammaPx;
    vector<double> AGATA_GammaPy;
    vector<double> AGATA_GammaPz;
    vector<double> AGATA_GammaE;
    vector<double> AGATA_OrigBetaX;
    vector<double> AGATA_OrigBetaY;
    vector<double> AGATA_OrigBetaZ;
    vector<double> AGATA_OrigBetaMag;
    double EAgata;
    std::vector<double> ELab;
    std::vector<double> Ex;
    std::vector<double> Ecm;
    std::vector<double> RawEnergy;
    std::vector<double> ELoss_Al;
    std::vector<double> ELoss_Target;
    std::vector<double> ELoss;
    std::vector<double> ThetaLab;
    std::vector<double> PhiLab;
    std::vector<double> ThetaCM;
    std::vector<double> HeavyThetaLab;
    std::vector<double> HeavyPhiLab;
    std::vector<double> HeavyPosXAtWindow;
    std::vector<double> HeavyPosYAtWindow;
    std::vector<double> HeavyPosXAtMUST2;
    std::vector<double> HeavyPosYAtMUST2;
    std::vector<double> HeavyAngX;
    std::vector<double> HeavyAngY;
    
    double WindowDistance = 681.33;
    double MUST2InnerDistance = 391.4;
  
    //For use in simulations
    double OriginalELab;
    double OriginalThetaLab;
    double BeamEnergy;

    NPL::Reaction reaction;
    NPL::Reaction reactionTest;
    //NPL::Reaction *reaction;
    //	Energy loss table: the G4Table are generated by the simulation
    NPL::EnergyLoss LightTarget;
    NPL::EnergyLoss LightAl;
    NPL::EnergyLoss LightSi;
    NPL::EnergyLoss LightAu;
    NPL::EnergyLoss BeamTarget;
    NPL::EnergyLoss HeavyTarget;

    double TargetThickness ;
    double WindowsThickness;
    // Beam Energy
    double OriginalBeamEnergy ; // AMEV
    double FinalBeamEnergy; 

    // Beam Position
    double XBeam;
    double YBeam;

    // intermediate variable
    TVector3 BeamDirection;
    TVector3 BeamImpact;
    TRandom3 Rand;
    int Run;
    int DetectorNumber;
    double ThetaNormalTarget;
    double ThetaM2Surface;
    double ThetaMGSurface;
    double elab_tmp;
    double thetalab_tmp;
    double philab_tmp;
    double Si_E_M2;
    double CsI_E_M2;
    double Energy;
    #define GATCONF_SIZE 16
    unsigned int GATCONF_Counter[GATCONF_SIZE];
    string GATCONF_Label[GATCONF_SIZE] = { 
      "MUVI1",
      "MUVI2",
      "MUVI3",
      "",
      "CATS2%",
      "VAMOS%",
      "MUST2%",
      "",
      "",
      "VAMOS",
      "ORAGATA", 
    } ;
    
    double MG_T;
    double MG_E;
    int    MG_X;
    int    MG_Y;
    int    MG_D;

    double ThetaGDSurface ;
    std::vector<double> X ;
    std::vector<double> Y ;
    std::vector<double> Z ;
    int AHeavy;
    int ALight;
    double MHeavy;
    double MLight;

    // Vamos Branches
    unsigned long long int LTS;
    unsigned short T_FPMW_CATS1; 
    float T_FPMW_CATS2_C; 
    unsigned short T_FPMW_HF; 
    unsigned short T_CATS2_HF;
    unsigned short T_MUGAST_FPMW;
    unsigned short T_FPMW_CATS2;
    float T_FPMW_HF_C; 
    float IC[12]; 
    unsigned short ICRawPUMult01[7]; 

    float DC0_X;
    float DC0_Y;

    float DC1_X;
    float DC1_Y;

    float DC2_X;
    float DC2_Y;

    float DC3_X;
    float DC3_Y;

    //float Xf;
    float Tf;

    float Brho;
    float Theta;
    float Phi;
    float Path;

    unsigned short EWIRE_1_1;
    unsigned short EWIRE_1_2;
    unsigned short EWIRE_2_1;
    unsigned short EWIRE_2_2;

    int MW_Nr;
    float MW_T[1000];
    float MWT[1000];
    unsigned short MW_N[1000];
    unsigned long long int AGAVA_VAMOSTS;

    // Vamos Calculated
    double BrhoRef;
    double mT;    
    double mV;   
    double mD;    
    double mBeta;
    double mGamma;
    double mM_Q;  
    double mM;    
    double mE;    
    double mdE;   

    // Agata branches
    double agata_zShift;
    /*
    unsigned long long int TStrack;
    int nbHits;
    int nbTrack;
    float *trackE  = new float(1000);
    float *trackX1 = new float(1000);
    float *trackY1 = new float(1000);
    float *trackZ1 = new float(1000);
    float *trackT  = new float(1000);
    int *trackCrystalID = new int(1000);
    */
    int nbAdd;
    unsigned long long int TSHit;
    float AddE[1000] ;
    float AddX[1000] ;
    float AddY[1000] ;
    float AddZ[1000] ;

    double EventWeight; //double or uns long long int?

    //
    double dE;
    double dTheta;
    // Branches and detectors
    TMust2Physics* M2;
    TMugastPhysics* MG;
    //TCATSPhysics* CATS;
    TModularLeafPhysics* ML;
    //TPlasticPhysics* VM;

    bool warning=true;
    

    //Simulation Stuff
    bool isSim;
    bool isPhaseSpace;
    bool excludePoor;
    bool writetoscreen;
    TInitialConditions* Initial;
    TInteractionCoordinates* Interaction;
    TReactionConditions* ReactionConditions;
    bool hitVAMOS = false;

    // Beam object
    NPL::Beam* Beam;

    unsigned int GATCONF_MASTER;

    unsigned long long int count ;




  /******** FROM IZANON ********/
  /******** FROM IZANON ********/
  /******** FROM IZANON ********/
  /******** FROM IZANON ********/

  private:
  //double EDC;
	std::vector<int> MGDet;
	std::vector<int> MGXStrip;
	std::vector<int> MGYStrip;
  std::vector<double> EDC_VAMOS;
  std::vector<double> EDC_MUGAST;
  std::vector<double> beta_VAMOS;
  std::vector<double> beta_MUGAST;
  unsigned short ToF;

  std::vector<double> gammaEDC;
  std::vector<double> trackEDC;
	std::vector<double> vamosEDC;
	std::vector<double> goldEDC;
  double recoilBeta;
  std::vector<double> ThetaGR;
  TVector3 recoilDir;
  TVector3 gammaDir;
  std::vector<double> ThetaG;
	double radius;
  std::vector<double> PhiG;	
  std::vector<double> ExCalib;
  std::vector<double> ExTmp;
  std::vector<double> ThetaGamma;
	double Theta4;
	double Phi4;
	double Energy4;
//  std::vector<double> Theta4;
//  std::vector<double> Phi4;
  std::vector<double> Theta3;
  std::vector<double> Phi3;	
  std::vector<int>	NTelescope;
  std::vector<int>	NXStrip;
   std::vector<int>	NYStrip;
    
	NPL::Reaction tmp_reaction;
  //	Energy loss table: the G4Table are generated by the simulation
  NPL::EnergyLoss HeavyAu;
  NPL::EnergyLoss* BeamWindow;
  NPL::EnergyLoss* LightWindow;

  //DC Optimization
  TH2I *mEDC_PhiAgata;
  TH2I *mEDC_OffsetAgata;
  TH2I *backingThick;
//TH3I *beamPos[7];
//TH3I *kineShift;

  double FrontDeformation ;
  double FrontRadius ;
  double AuThickness;
  // Beam Energy

  // intermediate variable
  double thetalab_2 ;
  double E4_tmp;
   
  double DSSD_T;
  int TelNbr;
  bool m_ValidEvent;

  // Vamos Branches
  unsigned long long int VTS;
  float Xf;

  // Vamos Calculated
  double mT1;    
  double mD2;    
  double mQ;    
  double mMrec;   
    
  // Agata branches
    unsigned long long int TStrack;
    //int nbHits;
    /*int nbTrack;
    float *trackE  = new float(1000);
    float *trackX1 = new float(1000);
    float *trackY1 = new float(1000);
    float *trackZ1 = new float(1000);
    float *trackT  = new float(1000);
    int *trackCrystalID = new int(1000);
	*/
	int nbTrack;
  float trackE[1000];
  float trackX1[1000];
  float trackY1[1000];
  float trackZ1[1000];
  float trackT[1000];
  Int_t trackCrystalID[1000]; 

  // Hit info for tracking opt
  Int_t nbHits;
  float hitE[1000] ;
  float hitX[1000] ;
  float hitY[1000] ;
  float hitZ[1000] ;
  Int_t hitId[1000];
	Int_t hitSg[1000];
		//
  // Branches and detectors
  TCATSPhysics* CATS;


};
#endif
 
